/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.Utilities;

import org.spongycastle.crypto.AsymmetricBlockCipher;
import org.spongycastle.crypto.CipherParameters;
import org.spongycastle.crypto.engines.RSAEngine;
import org.spongycastle.crypto.params.RSAKeyParameters;

import java.math.BigInteger;

/**
 * @author André Szdzuy
 */
public class EgkRSAEngine implements AsymmetricBlockCipher {
    private RSAEngine engine;
    private CipherParameters param;
    private boolean forEncryption;

    public EgkRSAEngine() {
    }

    public void init(boolean forEncryption, CipherParameters param) {
        if (engine == null) {
            engine = new RSAEngine();
            this.param = param;
            this.forEncryption = forEncryption;
        }

        engine.init(forEncryption, param);
    }

    public int getInputBlockSize() {
        return engine.getInputBlockSize();
    }

    public int getOutputBlockSize() {
        return engine.getOutputBlockSize();
    }

    public byte[] processBlock(byte[] in, int inOff, int inLen) {
        if (engine == null) {
            throw new IllegalStateException("RSA engine not initialised");
        } else {
            if (forEncryption) {
                BigInteger a = new BigInteger(engine.processBlock(in, inOff, inLen));
                BigInteger b = ((RSAKeyParameters) param).getModulus().subtract(a);
                return a.min(b).toByteArray();
            } else {
                byte[] sByte = engine.processBlock(in, 0, inLen);
                BigInteger s = new BigInteger(sByte);
                if (s.testBit(0)) {
                    s = ((RSAKeyParameters) param).getModulus().subtract(s);
                }
                return s.toByteArray();
            }
        }
    }
}
