/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.test;

import de.gematik.security.Utilities.CardFileSystem;
import de.gematik.security.impl.HealthcardContext;
import de.gematik.security.iso7816.Iso7816UnsuccessfulExecutionException;
import de.gematik.security.iso9564.PinBlockException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Test;

import javax.smartcardio.CardException;
import javax.smartcardio.CardTerminal;
import javax.smartcardio.CardTerminals;
import javax.smartcardio.TerminalFactory;
import java.io.IOException;
import java.util.List;

/**
 * @author André Szdzuy
 */
public class NfcConnectTest extends NfcBaseTest {
    private static final Log LOG = LogFactory.getLog(NfcConnectTest.class);

    @Test
    public void connectCard() throws CardException, Iso7816UnsuccessfulExecutionException, PinBlockException, IOException {
        LOG.info("connectCard() - only before first test is executed");

        try {
            while (true) {
                CardTerminals cardTerminals = TerminalFactory.getDefault().terminals();
                LOG.info("Please insert card!");
                cardTerminals.waitForChange();
                List<CardTerminal> list = cardTerminals.list(CardTerminals.State.ALL);
                for (int i = 0; i < list.size(); i++) {
                    CardTerminal item = list.get(i);
                    if (item != null && item.isCardPresent()) {
                        cardTerminalName = item.getName();
                        LOG.info("Connect to card terminal " + cardTerminalName);
                        nfcCard = item.connect("*");
                        break;
                    }
                }

                if (nfcCard != null) {
                    LOG.info("Open secure channel");
                    healthcardContext = new HealthcardContext(nfcCard.openLogicalChannel(), true);
                    if (!healthcardContext.isSMCB()) {
                        healthcardContext.secureChannel("123123");
                        LOG.info("Pace established");
                    }
                    healthcardContext.verify("123456");
                    LOG.info("PIN Home / PIN.CH / PIN.SMC verified");

                    if (healthcardContext.isEGK()) {
//                        //SELECT DF.HCA
//                        healthcardContext.selectADF("D27600000102");
//
//                        //SELECT DF.HCA/DF.NFD
//                        healthcardContext.selectADF("D27600014407");
//                        healthcardContext.verify(3, "123456");
//                        LOG.info("MRPIN.NFD verified");
//
//                        //SELECT DF.HCA/DF.AMTS
//                        healthcardContext.selectADF("D2760001440C");
//                        healthcardContext.verify(12, "123456");
//                        LOG.info("MRPIN.AMTS verified");

                        cfs = CardFileSystem.getEGKInstance();
                    } else if (healthcardContext.isHBA()) {
                        cfs = CardFileSystem.getHBAInstance();
                    } else if (healthcardContext.isSMCB()) {
                        cfs = CardFileSystem.getSMCBInstance();
                    } else {
                        throw new CardException("Inserted card is neither EGK, HBA nor SMC-B.");
                    }

                    break;
                }
            }
        } catch (Exception e) {
            LOG.error(e.getMessage());
            throw e;
        }
    }

}
